@@ -18,6 +18,8 @@
 //! [IntervalSet::write_dot](struct.IntervalSet.html#method.write_dot)).
 //! You can disable this feature using `cargo build --no-default-features`,
 //! in that case the crate supports `no_std` environments.
+//!
+//! Since version 0.0.5, this crate supports serialization/deserialization (use optional feature `serde`).
 
 // TODO:
 // - deletion
@@ -42,6 +44,14 @@ use core::ops::{Range, RangeFull, RangeInclusive, RangeBounds, Bound};
 use core::fmt::{self, Debug, Display, Formatter};
 #[cfg(feature = "dot")]
 use std::io::{self, Write};
+#[cfg(feature = "serde")]
+use core::marker::PhantomData;
+#[cfg(feature = "serde")]
+use serde::{Serialize, Serializer, Deserialize, Deserializer};
+#[cfg(feature = "serde")]
+use serde::ser::{SerializeTuple, SerializeSeq};
+#[cfg(feature = "serde")]
+use serde::de::{Visitor, SeqAccess};
 
 pub use iter::*;
 
@@ -93,6 +103,21 @@ impl<T: PartialOrd + Copy> Interval<T> {
     }
 }
 
+#[cfg(feature = "serde")]
+impl<T: PartialOrd + Copy + Serialize> Serialize for Interval<T> {
+    fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
+        (self.start, self.end).serialize(serializer)
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de, T: PartialOrd + Copy + Deserialize<'de>> Deserialize<'de> for Interval<T> {
+    fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+        let (start, end) = <(T, T)>::deserialize(deserializer)?;
+        Ok(Interval { start, end })
+    }
+}
+
 /// Trait for index types: used in the inner representation of [IntervalMap](struct.IntervalMap.html) and
 /// [IntervalSet](struct.IntervalSet.html).
 ///
@@ -160,6 +185,7 @@ impl_index!(u128);
 /// Default index type.
 pub type DefaultIx = u32;
 
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone)]
 struct Node<T: PartialOrd + Copy, V, Ix: IndexType> {
     interval: Interval<T>,
@@ -755,6 +781,110 @@ impl<T: PartialOrd + Copy + Debug, V: Debug, Ix: IndexType> Debug for IntervalMa
     }
 }
 
+#[cfg(feature = "serde")]
+impl<T, V, Ix> Serialize for IntervalMap<T, V, Ix>
+    where
+        T: PartialOrd + Copy + Serialize,
+        V: Serialize,
+        Ix: IndexType + Serialize,
+{
+    fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
+        // For some reason, Vec<Node> does not support serialization. Because of that we create a newtype.
+        struct NodeVecSer<'a, T, V, Ix>(&'a Vec<Node<T, V, Ix>>)
+            where
+                T: PartialOrd + Copy + Serialize,
+                V: Serialize,
+                Ix: IndexType + Serialize;
+
+        impl<'a, T, V, Ix> Serialize for NodeVecSer<'a, T, V, Ix>
+            where
+                T: PartialOrd + Copy + Serialize,
+                V: Serialize,
+                Ix: IndexType + Serialize,
+        {
+            fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
+                let mut seq = serializer.serialize_seq(Some(self.0.len()))?;
+                for node in self.0 {
+                    seq.serialize_element(node)?;
+                }
+                seq.end()
+            }
+        }
+
+        let mut tup = serializer.serialize_tuple(2)?;
+        tup.serialize_element(&NodeVecSer(&self.nodes))?;
+        tup.serialize_element(&self.root)?;
+        tup.end()
+    }
+}
+
+// For some reason, Vec<Node> does not support deserialization. Because of that we create a newtype.
+#[cfg(feature = "serde")]
+struct NodeVecDe<T: PartialOrd + Copy, V, Ix: IndexType>(Vec<Node<T, V, Ix>>);
+
+#[cfg(feature = "serde")]
+impl<T: PartialOrd + Copy, V, Ix: IndexType> NodeVecDe<T, V, Ix> {
+    fn take(self) -> Vec<Node<T, V, Ix>> {
+        self.0
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de, T, V, Ix> Deserialize<'de> for NodeVecDe<T, V, Ix>
+    where
+        T: PartialOrd + Copy + Deserialize<'de>,
+        V: Deserialize<'de>,
+        Ix: IndexType + Deserialize<'de>,
+{
+    fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+        struct NodeVecVisitor<T: PartialOrd + Copy, V, Ix: IndexType> {
+            marker: PhantomData<(T, V, Ix)>,
+        }
+
+        impl<'de, T, V, Ix> Visitor<'de> for NodeVecVisitor<T, V, Ix>
+        where
+            T: PartialOrd + Copy + Deserialize<'de>,
+            V: Deserialize<'de>,
+            Ix: IndexType + Deserialize<'de>,
+        {
+            type Value = NodeVecDe<T, V, Ix>;
+
+            fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                formatter.write_str("a sequence of Node<T, V, Ix>")
+            }
+
+            fn visit_seq<A: SeqAccess<'de>>(self, mut seq: A) -> Result<Self::Value, A::Error> {
+                let mut values = Vec::new();
+                while let Some(value) = seq.next_element()? {
+                    values.push(value);
+                }
+                Ok(NodeVecDe(values))
+            }
+        }
+
+        let visitor = NodeVecVisitor {
+            marker: PhantomData,
+        };
+        deserializer.deserialize_seq(visitor)
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de, T, V, Ix> Deserialize<'de> for IntervalMap<T, V, Ix>
+where
+    T: PartialOrd + Copy + Deserialize<'de>,
+    V: Deserialize<'de>,
+    Ix: IndexType + Deserialize<'de>,
+{
+    fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+        let (node_vec, root) = <(NodeVecDe<T, V, Ix>, Ix)>::deserialize(deserializer)?;
+        Ok(IntervalMap {
+            nodes: node_vec.take(),
+            root,
+        })
+    }
+}
+
 /// Set with interval keys (ranges `x..y`). Newtype over `IntervalMap<T, ()>`.
 ///
 /// See [IntervalMap](struct.IntervalMap.html) for more information.
@@ -933,6 +1063,29 @@ impl<T: PartialOrd + Copy + Debug, Ix: IndexType> Debug for IntervalSet<T, Ix> {
     }
 }
 
+#[cfg(feature = "serde")]
+impl<T, Ix> Serialize for IntervalSet<T, Ix>
+where
+    T: PartialOrd + Copy + Serialize,
+    Ix: IndexType + Serialize,
+{
+    fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
+        self.inner.serialize(serializer)
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de, T, Ix> Deserialize<'de> for IntervalSet<T, Ix>
+where
+    T: PartialOrd + Copy + Deserialize<'de>,
+    Ix: IndexType + Deserialize<'de>,
+{
+    fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+        let inner = <IntervalMap<T, (), Ix>>::deserialize(deserializer)?;
+        Ok(IntervalSet { inner })
+    }
+}
+
 /// Macros for [IntervalMap](struct.IntervalMap.html) creation.
 /// ```rust
 /// #[macro_use] extern crate iset;
